// (c) Max Kresch, licensed under GPLv3, 2008.
//     See LICENSE for further details.

#ifndef DOS_H
#define DOS_H

#include <vector>
#include "Matrix.H"
#include "Vector.H"

class Dos {
  public:
    inline Dos(void):N(0),x(new double[0]),y(new double[0]) { }
    inline Dos(const int N,const double* X,const double*Y):
      N(N),x(new double[N]),y(new double[N]) {  
      for(int i=0;i<N;i++){ x[i]=X[i];y[i]=Y[i]; }
    }
    inline Dos(const Dos& D): N(D.N),x(new double[D.N]),y(new double[D.N]){
      for(int i=0;i<N;i++){ x[i]=D.x[i]; y[i]=D.y[i]; }
    }
    inline Dos(const int N): N(N),x(new double[N]),y(new double[N]) { }
    inline ~Dos(void){ delete[] x; delete[] y; }
    inline void normalize(double d){  
      double sum(0);
      for(int i=0;i<N;i++){ sum += y[i]; }
      for(int i=0;i<N;i++){ y[i] *= d/sum; }
    }
    inline void normalize(void){  
      double sum(0);
      double de = x[1]-x[0];
      for(int i=0;i<N;i++){ sum += y[i]; }
      for(int i=0;i<N;i++){ y[i] /= de*sum; }
    }
    inline Dos operator=(const Dos& D){ 
      N = D.N;
      delete[] x;
      delete[] y;
      x =new double[N];
      y =new double[N];
      for(int i=0;i<N;i++){ x[i]=D.x[i]; y[i]=D.y[i]; }
      return *this;
    }
    inline Dos operator+(Dos D){
      Dos res(N);
      for(int i=0;i<N;i++){ res.x[i] = x[i]; res.y[i] = y[i]+D.y[i]; }
      return res;
    }
    inline Dos operator*(double d){ 
      Dos res(N);
      for(int i=0;i<N;i++){ res.x[i]=x[i]; res.y[i]= d*y[i];}
      return res;
    }
    inline Dos rescale(const double d) const{ 
      Dos res(N);
      for(int i=0;i<N;i++){ res.x[i]= d*x[i]; res.y[i]=y[i]; }
      res.normalize();
      return res;
    }
    inline Dos rebin(const double* X) const{
      Dos res(N);
      for(int i=0;i<N;i++){ res.x[i]=X[i]; res.y[i] = 0.0; }
      int i(1);
      int j(1);
      while( i < N && j < N ){
        if(x[i] < res.x[j]){ i++; }
        else{ 
          res.y[j] = y[i-1]*(x[i] - res.x[j]) + y[i]*(res.x[j]-x[i-1]);
          j++;
        }
      }
      res.normalize();
      return res;
    } 
    inline Matrix getBroadeningMatrix(const double Q){
      normalize(1.0);
      Matrix D(N);
      for(int i=0;i<N*N;i++){D.elts[i]=0.0;}
//Assumes that first slot of DOS is 0 
      D.elts[0] = 1.0;
      for(int r=1;r<N;r++){
        double sum = 0;
        for(int c=0;c<N;c++){
          D.elts[r*N+c] = ( x[c]*x[c] )/
           ( (x[c]/Q)*(x[c]/Q) + (x[r]-x[c]*x[c]/x[r])*(x[r]-x[c]*x[c]/x[r]) );
          sum += D.elts[r*N+c];
        }
        for(int c=0;c<N;c++){ D.elts[r*N+c] /= sum; }
      }
      return D;
    }

    inline Dos broaden(const double Q){ 
      normalize(1.0);
      Matrix D(N);
      for(int i=0;i<N*N;i++){D.elts[i]=0.0;}
      Vector Y(N);
//Assumes that first slot of DOS is 0 
      D.elts[0] = 1.0;
      Y.elts[0] = y[0];
      for(int r=1;r<N;r++){
        Y.elts[r] = y[r];
        double sum = 0;
        for(int c=0;c<N;c++){
          D.elts[r*N+c] = ( x[c]*x[c] )/
           ( (x[c]/Q)*(x[c]/Q) + (x[r]-x[c]*x[c]/x[r])*(x[r]-x[c]*x[c]/x[r]) );
          sum += D.elts[r*N+c];
        }
        for(int c=0;c<N;c++){ D.elts[r*N+c] /= sum; }
      }
      Dos res(N,x,(D*Y).elts);
      normalize();
      res.normalize();
      return res;
    }
    inline void dump(void) const{   
      printf("|   x       y    |\n");
      for(int i=0;i<N;i++){printf("|% 7.3f % 7.3f |\n",x[i],y[i]);}
    }
    inline void fromText(const char * filename){ 
      std::vector<double> tmpX;
      std::vector<double> tmpY;
      char *tok = NULL;
      char curr_line[100];
      char delim[] = " ";
      FILE* io=fopen(filename,"r");
      while(fgets(curr_line,100,io)){
        tmpX.push_back( atof( strtok(curr_line,delim) ));
        tmpY.push_back( atof( strtok(NULL,delim) ));
      }
      fclose(io);
      N = tmpX.size();
      delete[] x;
      delete[] y;
      x = new double[N];
      y = new double[N];
      for(int i=0;i<N;i++){ x[i]=tmpX[i]; y[i]=tmpY[i]; }
      return;
    }
    inline void toText(const char * filename){ 
      FILE* io=fopen(filename,"wc");
      for(int i=0;i<N;i++){ fprintf(io,"%lf %lf\n",x[i],y[i]); }
     fclose(io);
    }
  public:
    int N;
    double* x;
    double* y;
};

#endif // DOS_H
